using System;
using System.IO;
using TSLab.DataSource;

namespace FinInfoCache.Common
{
    public static class FinInfoCacheHelper
    {
        public static void WriteToStream(this FinInfoBlock fi, BinaryWriter bw)
        {
            bw.Write(fi.LastUpdate.Ticks);
            bw.Write(fi.Ask);
            bw.Write(fi.Bid);
            bw.Write(fi.BuyCount);
            bw.Write(fi.BuySqty);
            bw.Write(fi.SellCount);
            bw.Write(fi.SellSqty);
            bw.Write(fi.TheoriticPrice);
            bw.Write(fi.Volatility);
        }

        public static void WriteToStream(this FinInfo fi, BinaryWriter bw)
        {
            //     .       .
            // bw.Write(fi.LastUpdate.ToUniversalTime().Ticks);
            bw.Write(DateTime.UtcNow.Ticks);    //     ,    .
            bw.Write(fi.Ask ?? 0);
            bw.Write(fi.Bid ?? 0);
            bw.Write(fi.BuyCount ?? 0);
            bw.Write(fi.BuySqty ?? 0);
            bw.Write(fi.SellCount ?? 0);
            bw.Write(fi.SellSqty ?? 0);
            bw.Write(fi.TheoriticPrice ?? 0);
            bw.Write(fi.Volatility ?? 0);
            bw.Write(fi.OpenInterest ?? 0);
        }

        public static void ReadFromStream(this FinInfoBlock fi, long rec, BinaryReader br)
        {
            br.BaseStream.Seek(FininfoCache.HeaderSize + rec * FininfoCache.RecSize, SeekOrigin.Begin);
            
            var utcTime = new DateTime(br.ReadInt64(), DateTimeKind.Utc);
            //var moscowTime = TimeZoneInfo.ConvertTimeFromUtc(utcTime,
            //                                                 TimeZoneInfo.FindSystemTimeZoneById("Russian Standard Time"));
            fi.LastUpdate = utcTime;
            fi.Ask = br.ReadDouble();
            fi.Bid = br.ReadDouble();
            fi.BuyCount = br.ReadDouble();
            fi.BuySqty = br.ReadDouble();
            fi.SellCount = br.ReadDouble();
            fi.SellSqty = br.ReadDouble();
            fi.TheoriticPrice = br.ReadDouble();
            fi.Volatility = br.ReadDouble();
            fi.OpenInterest = br.ReadDouble();
        }

        public static FinInfoBlock ReadFromStream(long rec, BinaryReader br)
        {
            var fiBlock = new FinInfoBlock();
            fiBlock.ReadFromStream(rec, br);

            return fiBlock;
        }
    }
}